# -*- coding: utf-8 -*-
# xbutil GUI Device Manager Module
# Developers: Jim Wu, Annie Ren
# MIT License
# Copyright (c) 2020-2021 xbutil GUI Developers

import tkinter as tk
from tkinter import ttk, Toplevel, scrolledtext, messagebox
from tksheet import Sheet
import subprocess
import os
import datetime
import re

from xbutil_gui import VERSION, LABEL_WIDTH

SHEET_DEVICES_TOTAL_ROWS = 200
SHEET_DEVICES_TOTAL_COLS = 9
SHEET_DEVICES_HOST_COL = 0
SHEET_DEVICES_PCIE_BDF_COL = 1
SHEET_DEVICES_DEVICE_COL = 2
SHEET_DEVICES_PCIE_CAP_SPEED_COL = 3
SHEET_DEVICES_PCIE_STA_SPEED_COL = 4
SHEET_DEVICES_PCIE_CAP_WIDTH_COL = 5
SHEET_DEVICES_PCIE_STA_WIDTH_COL = 6
SHEET_DEVICES_PCIE_WR_BW_COL = 7
SHEET_DEVICES_PCIE_RD_BW_COL = 8


class DeviceManager:
    def __init__(self):
        self.selected_cluster_name = None
        self.pcie_dict = {}
        self.alveo_spec_dict = None

        # members for GUI
        self.label_cluster_name = None
        self.window_devman = None
        self.sheet_devices = None
        self.button_flash = None
        self.button_validate = None
        self.sheet_last_row = 1
        self.sheet_data_shadow = [{}] * SHEET_DEVICES_TOTAL_ROWS
        self.root_window_pointer = None
        self.window_log = None


    def validate_devices(self):
        # create a log widget if needed
        if self.window_log is None or not tk.Toplevel.winfo_exists(self.window_log):
            self.window_log = Toplevel(self.root_window_pointer)
            self.window_log.title('xbutil GUI ' + VERSION + ' - validate log')
            self.window_log.geometry('800x500+800+350')
            self.window_log.rowconfigure(0, weight=1)
            self.window_log.columnconfigure(0, weight=1)
            cur_grid_row = 0

            self.text_log = scrolledtext.ScrolledText(self.window_log)
            self.text_log.grid(row=cur_grid_row, column=0, sticky='nsew')
            self.text_log.update_idletasks()

        re_ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
        re_pcie_wrbw = re.compile('FPGA write bandwidth =\s+(.*) MB')
        re_pcie_rdbw = re.compile('FPGA read bandwidth =\s+(.*) MB')
        re_platform = re.compile('Platform\s+:\s+(.*)')
        dash_line = '-' * 60 + '\n'
        for row in range(1, self.sheet_last_row):
            host = self.sheet_data_shadow[row]['host']
            bdf = self.sheet_data_shadow[row]['bdf']
            self.text_log.insert(tk.END, dash_line)
            self.text_log.insert(tk.END, 'Validating device ' + bdf + \
                                         ' on host ' + host + '\n')
            self.text_log.insert(tk.END, dash_line)

            xbutil_command = '/opt/xilinx/xrt/bin/xbutil --new validate -d ' + bdf
            if host == 'localhost':
                command = xbutil_command
            else:
                command = 'ssh ' + host + ' ' + xbutil_command

            p = subprocess.Popen(command, stdout=subprocess.PIPE, 
                                 stderr=subprocess.STDOUT, shell=True)
            for line in iter(lambda: p.stdout.readline(), b''):
                # print(line.decode('utf-8'))
                line = re_ansi_escape.sub('', line.decode('utf-8'))
                m = re_pcie_wrbw.search(line)
                if m:
                    self.pcie_dict[host][bdf]['pcie_wrbw'] = float(m.group(1))/1000
                m = re_pcie_rdbw.search(line)
                if m:
                    self.pcie_dict[host][bdf]['pcie_rdbw'] = float(m.group(1))/1000
                m = re_platform.search(line)
                if m:
                    self.pcie_dict[host][bdf]['device'] = m.group(1)

                self.text_log.insert(tk.END, line)
                self.text_log.yview(tk.END)
                self.text_log.update_idletasks()

            # finished current bdf. Update metrics
            self.text_log.insert(tk.END, dash_line)
            self.text_log.insert(tk.END, 'Completed validating ' + bdf + \
                                         ' on host' + host + '\n')
            self.text_log.insert(tk.END, dash_line)

            self.sheet_devices.set_cell_data(row, SHEET_DEVICES_DEVICE_COL,
                self.pcie_dict[host][bdf]['device'])
            self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_WR_BW_COL,
                '{:.2f}'.format(self.pcie_dict[host][bdf]['pcie_wrbw']))
            self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_RD_BW_COL,
                '{:.2f}'.format(self.pcie_dict[host][bdf]['pcie_rdbw']))
            self.sheet_devices.refresh()

        # finished all bdfs
        self.text_log.insert(tk.END, dash_line)
        self.text_log.insert(tk.END, 'Completed validating ' + str(self.sheet_last_row-1) +
                                     ' devices\n')
        self.text_log.insert(tk.END, dash_line)
        self.text_log.yview(tk.END)
        self.text_log.update_idletasks()

    def flash_devices(self):
        messagebox.showinfo("showinfo", "Coming soon")
        # host = 'host'
        # password = '12345678'
        # xbmgmt_cmd = ['echo', password, '|',
        #               'sudo', '-S', '/opt/xilinx/xrt/bin/xbmgmt', 'flash', '--update',
        #               '--shell', 'xilinx_u50_gen3x16_xdma_201920_3', '--force']
        # command = ['ssh', host] + xbmgmt_cmd
        # print(' '.join(command))
        # p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        # for l in iter(lambda: p.stdout.readline(), b''):
        #     print(l)
        #
        # print('INFO: Completed flashing all devices')

    def get_devices(self, sudo_password, hosts):
        # first save the password to a file with permission 0o600 to be secure
        password_file = os.path.expanduser("~") + '/.xbutil-gui-tmp'
        with open(password_file, 'w') as fh:
            os.chmod(password_file, 0o600)
            fh.write(sudo_password)

        lspci_cmd = 'sudo -S <<< $(cat ~/.xbutil-gui-tmp) lspci -d 10ee: -vv'
        self.pcie_dict = {}
        for host in hosts:
            self.pcie_dict[host] = {}
            if host == 'localhost':
                command = lspci_cmd
            else:
                command = 'ssh ' + host + ' ' + lspci_cmd

            #print('INFO: run {} on host {}'.format(command, host))
            p = subprocess.Popen(command, stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT, shell=True)
            lspci_out = p.stdout.read().decode('utf-8')
            lspci_lines = lspci_out.split('\n')
            first_line = True
            is_function1 = False
            for line in lspci_lines:
                if line.strip() == "":
                    continue

                if first_line and line.startswith('Password:'):
                    line = line[len('Password:'):]
                    first_line = False

                if not line[0].isspace():
                    fields = line.split()
                    bdf = fields[0]
                    if bdf[-1] == '1':
                        is_function1 = True
                        device_id = fields[-1]
                        self.pcie_dict[host][bdf] = {
                            'device_id': device_id,
                            'device': self.alveo_spec_dict['pcie_device_ids'][device_id],
                            'pcie_wrbw': '--',
                            'pcie_rdbw': '--'}
                    else:
                        is_function1 = False
                elif is_function1:
                    fields = line.split()
                    if fields[0] == 'LnkCap:':
                        self.pcie_dict[host][bdf]['LnkCapSpeed'] = fields[4][:-1]
                        self.pcie_dict[host][bdf]['LnkCapWidth'] = fields[6][:-1]
                    elif fields[0] == 'LnkSta:':
                        self.pcie_dict[host][bdf]['LnkStaSpeed'] = fields[2][:-1]
                        self.pcie_dict[host][bdf]['LnkStaWidth'] = fields[4][:-1]

            # finished processing all lines

        # finished processing all hosts
        self.display_devices()

    def display_devices(self):
        row = 1
        for host in self.pcie_dict.keys():
            for bdf in self.pcie_dict[host].keys():
                lnkcap_speed = self.pcie_dict[host][bdf]['LnkCapSpeed']
                lnksta_speed = self.pcie_dict[host][bdf]['LnkStaSpeed']
                lnkcap_width = self.pcie_dict[host][bdf]['LnkCapWidth']
                lnksta_width = self.pcie_dict[host][bdf]['LnkStaWidth']
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_HOST_COL, host)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_BDF_COL, bdf)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_DEVICE_COL,
                                                 self.pcie_dict[host][bdf]['device'])
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_CAP_SPEED_COL, lnkcap_speed)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_STA_SPEED_COL, lnksta_speed)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_CAP_WIDTH_COL, lnkcap_width)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_STA_WIDTH_COL, lnksta_width)
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_WR_BW_COL,
                                                 self.pcie_dict[host][bdf]['pcie_wrbw'])
                self.sheet_devices.set_cell_data(row, SHEET_DEVICES_PCIE_RD_BW_COL,
                                                 self.pcie_dict[host][bdf]['pcie_rdbw'])

                if lnkcap_speed != lnksta_speed:
                    self.sheet_devices.highlight_cells(row, SHEET_DEVICES_PCIE_STA_SPEED_COL, bg='red')
                if lnkcap_width != lnksta_width:
                    self.sheet_devices.highlight_cells(row, SHEET_DEVICES_PCIE_STA_WIDTH_COL, bg='red')

                self.sheet_data_shadow[row] = {'host': host, 'bdf': bdf}
                row = row + 1

        self.sheet_last_row = row

    def show_devman_window(self, root_window, cluster_name):
        self.selected_cluster_name = cluster_name
        if self.window_devman is not None and tk.Toplevel.winfo_exists(self.window_devman):
            self.label_cluster_name['text'] = self.selected_cluster_name
            return

        self.root_window_pointer = root_window
        self.window_devman = Toplevel(root_window)
        self.window_devman.title('xbutil GUI ' + VERSION + ' - Device Manager')
        self.window_devman.geometry('1200x400+20+150')
        # auto resize the row for sheet
        self.window_devman.grid_rowconfigure(1, weight=1)
        self.window_devman.columnconfigure(0, weight=0, minsize=150)
        self.window_devman.columnconfigure(1, weight=0, minsize=150)
        self.window_devman.columnconfigure(2, weight=0, minsize=150)
        self.window_devman.columnconfigure(3, weight=0, minsize=150)
        self.window_devman.columnconfigure(4, weight=0, minsize=150)
        self.window_devman.columnconfigure(5, weight=1, minsize=150)

        cur_grid_row = 0
        label_cluster = ttk.Label(self.window_devman, text="Cluster:",
                                  width=LABEL_WIDTH, anchor='e')
        label_cluster.grid(row=cur_grid_row, column=0, sticky='e')
        self.label_cluster_name = ttk.Label(self.window_devman,
                                            text=self.selected_cluster_name,
                                            width=40, anchor='w')
        self.label_cluster_name.grid(row=cur_grid_row, column=1, sticky='w')
        cur_grid_row = cur_grid_row + 1

        # sheet for all devices
        self.sheet_devices = Sheet(self.window_devman,
                              default_row_index="numbers",
                              total_rows=SHEET_DEVICES_TOTAL_ROWS,
                              total_columns=SHEET_DEVICES_TOTAL_COLS
                              )
        self.sheet_devices.enable_bindings(("single_select",  # "single_select" or "toggle_select"
                                       "drag_select",  # enables shift click selection as well
                                       "column_drag_and_drop",
                                       "row_drag_and_drop",
                                       "row_select",
                                       "column_width_resize",
                                       "double_click_column_resize",
                                       "arrowkeys",
                                       "right_click_popup_menu",
                                       "rc_select",
                                       "copy",
                                       "cut",
                                       "paste",
                                       "delete",
                                       "undo",
                                       "edit_cell"))
        self.sheet_devices.grid(row=cur_grid_row, columnspan=6, sticky='nswe')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_HOST_COL, 'Host')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_BDF_COL, 'PCIe BDF')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_DEVICE_COL, 'Device')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_CAP_SPEED_COL, 'LnkCap Spd')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_STA_SPEED_COL, 'LnkSta Spd')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_CAP_WIDTH_COL, 'LnkCap xW')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_STA_WIDTH_COL, 'LnkSta xW')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_WR_BW_COL, 'PCIeWr GB/s')
        self.sheet_devices.set_cell_data(0, SHEET_DEVICES_PCIE_RD_BW_COL, 'PCIeRd GB/s')
        self.sheet_devices.column_width(column=SHEET_DEVICES_HOST_COL, width=150)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_BDF_COL, width=100)
        self.sheet_devices.column_width(column=SHEET_DEVICES_DEVICE_COL, width=250)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_CAP_SPEED_COL, width=75)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_STA_SPEED_COL, width=75)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_CAP_WIDTH_COL, width=75)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_STA_WIDTH_COL, width=75)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_WR_BW_COL, width=75)
        self.sheet_devices.column_width(column=SHEET_DEVICES_PCIE_RD_BW_COL, width=75)

        cur_grid_row = cur_grid_row + 1

        self.button_validate = ttk.Button(self.window_devman, text="Validate",
                                          command=self.validate_devices)
        self.button_validate.grid(row=cur_grid_row, column=1, pady=10)
        self.button_flash = ttk.Button(self.window_devman, text="Flash",
                                       command=self.flash_devices)
        self.button_flash.grid(row=cur_grid_row, column=2, pady=10)






# -*- coding: utf-8 -*-

import tkinter as tk
from tkinter import ttk, messagebox, simpledialog
from tksheet import Sheet
import subprocess
import os
import json
import shutil
import datetime
import argparse
from pathlib import Path

from xbutil_gui.xbutil_top import XbutilTop
from xbutil_gui.plot_metrics import PlotMetrics
from xbutil_gui.device_manager import DeviceManager
from xbutil_gui.xbutil_handler import get_devices_compute_units, \
                                      get_xbutil_dump, get_devices_from_lspci
from xbutil_gui import VERSION, LABEL_WIDTH, COMBO_WIDTH, STATUS_CODES, \
                DEFAULT_XBUTIL_REFRESH_INTERVAL, __resource_path__, __icon__, \
                SHEET_TOTAL_ROWS, SHEET_TOTAL_COLS,  \
                SHEET_HOST_COL, SHEET_DEVICE_COL, SHEET_CU_COL, SHEET_CU_STATUS_COL, \
                SHEET_CU_USAGE_COL, SHEET_POWER_COL, SHEET_TEMP_COL, \
                SHEET_LAST_UPDATED_COL


# interval in seconds between xbutil json dumps
auto_refresh_plot_seconds = 0
xbutil_top = XbutilTop()
plot_metrics = PlotMetrics()
device_manager = DeviceManager()

shadow_sheet_hosts = ['' for i in range(SHEET_TOTAL_ROWS)]
shadow_sheet_device_id_names = ['' for i in range(SHEET_TOTAL_ROWS)]
pause_sheet = 0

def get_selected_host_device():
    global shadow_sheet_hosts, shadow_sheet_device_id_names

    sheet_selected = sheet_cluster.get_currently_selected()
    if len(sheet_selected) == 0:
        messagebox.showinfo("showinfo", "Please select a cell or row on the sheet first.")
        return -1, None, None

    if str(sheet_selected[0]) == 'row':
        selected_row = sheet_selected[1]
    else:
        selected_row = sheet_selected[0]

    selected_host = shadow_sheet_hosts[selected_row]
    selected_device_id_name = shadow_sheet_device_id_names[selected_row]
    if len(selected_device_id_name) == 0:
        messagebox.showinfo("showinfo", "Please wait until devices are scanned.")
        return -2, None, None

    return 0, selected_host, selected_device_id_name


def show_top_window():
    status, selected_host, selected_device_id_name = get_selected_host_device()
    if status < 0:
        return

    xbutil_top.show_top_window(root_window, selected_host, selected_device_id_name)


def show_plot_window():
    status, selected_host, selected_device_id_name = get_selected_host_device()
    if status < 0:
        return

    plot_metrics.show_plot_window(root_window, selected_host, selected_device_id_name)

def show_devman_window():
    sudo_password = tk.simpledialog.askstring("Password", "Enter Sudo password:", show='*')
    selected_cluster = combo_cluster.current()
    selected_cluster_name = combo_cluster['values'][selected_cluster]

    device_manager.show_devman_window(root_window, selected_cluster_name)
    device_manager.get_devices(sudo_password, clusters[selected_cluster_name])


def toggle_pause_sheet():
    global pause_sheet
    if pause_sheet == 0:
        pause_sheet = 1
        button_pause_sheet['text'] = 'Resume'
    else:
        pause_sheet = 0
        button_pause_sheet['text'] = 'Pause'


###############################################################################
# root window
###############################################################################
root_window = tk.Tk()
root_window.geometry('1500x700+20+20')
root_window.title('Xilinx xbutil GUI ' + VERSION)
root_window_icon = tk.PhotoImage(file=str(__icon__))
root_window.iconphoto(True, root_window_icon)
root_window.columnconfigure(0, weight=0, minsize=150)
root_window.columnconfigure(1, weight=0, minsize=150)
root_window.columnconfigure(2, weight=0, minsize=150)
root_window.columnconfigure(3, weight=0, minsize=150)
root_window.columnconfigure(4, weight=0, minsize=150)
root_window.columnconfigure(5, weight=1, minsize=150)

cur_grid_row = 0
label_cluster = ttk.Label(root_window, text="Cluster", width=LABEL_WIDTH, anchor='w')
label_cluster.grid(row=cur_grid_row, column=0,  sticky='w', pady=10)
combo_cluster = ttk.Combobox(root_window, width=COMBO_WIDTH)
combo_cluster['values'] = []
combo_cluster.grid(row=cur_grid_row, column=1, columnspan=3, sticky='w', pady=10)
cur_grid_row = cur_grid_row + 1

# sheet for cluster
sheet_cluster = Sheet(root_window,
                      default_row_index="numbers",
                      total_rows=SHEET_TOTAL_ROWS,
                      total_columns=SHEET_TOTAL_COLS
                      )
sheet_cluster.enable_bindings(("single_select",  # "single_select" or "toggle_select"
                               "drag_select",  # enables shift click selection as well
                               "column_drag_and_drop",
                               "row_drag_and_drop",
                               #"column_select",
                               "row_select",
                               "column_width_resize",
                               "double_click_column_resize",
                               "arrowkeys",
                               #"row_height_resize",
                               #"double_click_row_resize",
                               "right_click_popup_menu",
                               "rc_select",
                               #"rc_insert_column",
                               #"rc_delete_column",
                               #"rc_insert_row",
                               #"rc_delete_row",
                               "copy",
                               "cut",
                               "paste",
                               "delete",
                               "undo",
                               "edit_cell"))
sheet_cluster.grid(row=cur_grid_row, columnspan=6, sticky='nswe')
root_window.grid_rowconfigure(cur_grid_row, weight=1)
sheet_cluster.set_cell_data(0, SHEET_HOST_COL, 'Host')
sheet_cluster.set_cell_data(0, SHEET_DEVICE_COL, 'Device ID::Shell')
sheet_cluster.set_cell_data(0, SHEET_CU_COL, 'Compute Unit (CU)')
sheet_cluster.set_cell_data(0, SHEET_CU_STATUS_COL, 'CU Status')
sheet_cluster.set_cell_data(0, SHEET_CU_USAGE_COL, 'CU Usage')
sheet_cluster.set_cell_data(0, SHEET_POWER_COL, 'P(W)')
sheet_cluster.set_cell_data(0, SHEET_TEMP_COL, 'T(C)')
sheet_cluster.set_cell_data(0, SHEET_LAST_UPDATED_COL, 'Last Updated')
sheet_cluster.column_width(column=SHEET_HOST_COL, width=150)
sheet_cluster.column_width(column=SHEET_DEVICE_COL, width=300)
sheet_cluster.column_width(column=SHEET_CU_COL, width=300)
sheet_cluster.column_width(column=SHEET_CU_STATUS_COL, width=60)
sheet_cluster.column_width(column=SHEET_CU_USAGE_COL, width=60)
sheet_cluster.column_width(column=SHEET_POWER_COL, width=50)
sheet_cluster.column_width(column=SHEET_TEMP_COL, width=50)
sheet_cluster.column_width(column=SHEET_LAST_UPDATED_COL, width=200)
cur_grid_row = cur_grid_row + 1
sheet_cluster_last_row = 0

# command buttons for selected host
button_top = ttk.Button(root_window, text="Top", command=show_top_window)
button_top.grid(row=cur_grid_row, column=1, pady=10)

button_plot = ttk.Button(root_window, text="Plot", command=show_plot_window)
button_plot.grid(row=cur_grid_row, column=2, pady=10)

button_manage_devices = ttk.Button(root_window, text="Manage", command=show_devman_window)
button_manage_devices.grid(row=cur_grid_row, column=3, pady=10)

button_pause_sheet = ttk.Button(root_window, text="Pause", command=toggle_pause_sheet)
button_pause_sheet.grid(row=cur_grid_row, column=4, pady=10)
cur_grid_row = cur_grid_row + 1

# global variables
cur_cluster_name = ""
auto_refresh_host_idx = 0
auto_refresh_sheet_row = 0
alveo_spec_dict = {}


def update_sheet_cluster(devices_compute_units, xbutil_dump_json, selected_cluster,
                         refresh_host):
    global auto_refresh_host_idx, auto_refresh_sheet_row, sheet_cluster_last_row
    global shadow_sheet_hosts, shadow_sheet_hosts, alveo_spec_dict, pause_sheet

    if pause_sheet == 1:
        return

    host_displayed = 0
    last_udpated = datetime.datetime.now().strftime("%m/%d/%Y, %H:%M:%S")

    for i_dn in range(len(devices_compute_units['device_id_names'])):
        #refresh_host = 'host' + str(i_dn)
        device_vbnv = devices_compute_units['device_vbnvs'][i_dn]
        device_id_name = devices_compute_units['device_id_names'][i_dn]
        last_metrics = plot_metrics.get_last_metrics(refresh_host, device_id_name)
        if xbutil_dump_json is not None:
            xbutil_top.generate_top_dict(xbutil_dump_json, refresh_host, device_id_name)
            plot_metrics.update_history(xbutil_dump_json, refresh_host, device_id_name)
        
        dev_displayed = 0
        for cu in devices_compute_units['compute_units'][i_dn]:
            if host_displayed == 0:
                sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_HOST_COL, refresh_host)
                sheet_cluster.highlight_rows([auto_refresh_sheet_row], bg='light sky blue' )
            else:
                sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_HOST_COL, '')
                sheet_cluster.dehighlight_rows([auto_refresh_sheet_row])

            host_displayed = 1

            if dev_displayed == 0:
                dev = device_id_name
                p_display = last_metrics[0]
                t_display = last_metrics[1]

                board = alveo_spec_dict["shell_board_lut"][device_vbnv]
                if t_display > alveo_spec_dict[board]['fpga_temp']['critical']:
                    sheet_cluster.highlight_cells(auto_refresh_sheet_row,
                                                  SHEET_TEMP_COL, bg='red')
                elif t_display > alveo_spec_dict[board]['fpga_temp']['warning']:
                    sheet_cluster.highlight_cells(auto_refresh_sheet_row,
                                                  SHEET_TEMP_COL, bg='yellow')
            else:
                dev = ''
                p_display = ''
                t_display = ''

            dev_displayed = 1
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_DEVICE_COL, dev)
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_CU_COL, cu['name'])
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_CU_STATUS_COL, cu['status'])
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_CU_USAGE_COL, cu['usage'])
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_POWER_COL, p_display)
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_TEMP_COL, t_display)
            sheet_cluster.set_cell_data(auto_refresh_sheet_row, SHEET_LAST_UPDATED_COL, last_udpated)
            # save host/dev_id_name into shadow varaibles
            shadow_sheet_hosts[auto_refresh_sheet_row] = refresh_host
            shadow_sheet_device_id_names[auto_refresh_sheet_row] = device_id_name
            auto_refresh_sheet_row = auto_refresh_sheet_row + 1


    sheet_cluster.refresh()
    auto_refresh_host_idx = auto_refresh_host_idx + 1
    if auto_refresh_host_idx == len(clusters[combo_cluster['values'][selected_cluster]]):
        auto_refresh_host_idx = 0
        
        if sheet_cluster_last_row > auto_refresh_sheet_row:
            # clear contents from previous full scan
            sheet_cluster.dehighlight_rows(range(auto_refresh_sheet_row, sheet_cluster_last_row+1))
            for r in range(auto_refresh_sheet_row, sheet_cluster_last_row+1):
                for c in range(SHEET_TOTAL_COLS):
                    sheet_cluster.set_cell_data(r, c, '')

        # udpate the last row count
        sheet_cluster_last_row = auto_refresh_sheet_row
        auto_refresh_sheet_row = 1


# get xbutil dump from each host in round robin fashion every XBUTIL_REFRESH_INTERVAL
def refresh_database(json_file):
    global auto_refresh_plot_seconds, auto_refresh_host_idx, cur_cluster_name, \
           auto_refresh_sheet_row, sheet_cluster_last_row

    selected_cluster = combo_cluster.current()
    selected_cluster_name = combo_cluster['values'][selected_cluster]
    if cur_cluster_name != selected_cluster_name:
        print('INFO: switch to new cluster', selected_cluster_name)
        auto_refresh_host_idx = 0
        auto_refresh_sheet_row = 1
        cur_cluster_name = selected_cluster_name
        sheet_cluster_last_row = SHEET_TOTAL_ROWS - 1

    refresh_host = clusters[selected_cluster_name][auto_refresh_host_idx]
    xbutil_dump_json,lspci_dict = get_xbutil_dump(json_file, host=refresh_host)

    if xbutil_dump_json is not None:
        devices_compute_units = get_devices_compute_units(xbutil_dump_json)
        update_sheet_cluster(devices_compute_units, xbutil_dump_json,
                             selected_cluster, refresh_host)
        xbutil_top.show_top_info()
        plot_metrics.plot_metrics(auto_refresh_plot_seconds)
    elif lspci_dict:
        devices_compute_units = get_devices_from_lspci(lspci_dict, alveo_spec_dict)
        update_sheet_cluster(devices_compute_units, xbutil_dump_json,
                             selected_cluster, refresh_host)
    else:
        # something wrong with current refresh host. Move on to the next host
        auto_refresh_host_idx = auto_refresh_host_idx + 1
        if auto_refresh_host_idx == len(clusters[selected_cluster_name]):
            auto_refresh_host_idx = 0

    # add refresh_database back to the eventloop
    auto_refresh_plot_seconds = auto_refresh_plot_seconds + DEFAULT_XBUTIL_REFRESH_INTERVAL
    root_window.after(DEFAULT_XBUTIL_REFRESH_INTERVAL*1000, refresh_database, json_file)


def main():
    global plot_metric, cur_cluster_name, clusters, auto_refresh_host_idx, \
           auto_refresh_sheet_row, alveo_spec_dict

    parser = argparse.ArgumentParser()
    parser.add_argument('--json-file', dest='json_file', default=None,
                        help='Specify a JSON file for getting the data')
    parser.add_argument('--plot-type', dest='plot_metric', default='power',
                        help='Specify plot type: power, temperature, or voltage')
    args = parser.parse_args()
    plot_metric = args.plot_metric
    #if args.json_file is None and shutil.which('xbutil') is None:
    #    print(STATUS_CODES['XRT_NOT_SETUP']['message'])
    #    exit(STATUS_CODES['XRT_NOT_SETUP']['code'])

    home = os.path.expanduser("~")
    user_config_file = home + '/xbutil-gui-config.json'
    default_config_file = __resource_path__ / 'xbutil-gui-config.json'
    cluster_names = []
    if Path(user_config_file).exists():
        config_file = Path(user_config_file)
    elif default_config_file.exists():
        config_file = default_config_file

    with open(config_file, 'r') as fp:
        xbutil_config_json = json.load(fp)

    alveo_spec_file = __resource_path__ / 'alveo-specifications.json'
    with open(alveo_spec_file, 'r') as fp:
        alveo_spec_dict = json.load(fp)

    device_manager.alveo_spec_dict = alveo_spec_dict

    clusters = xbutil_config_json.get('clusters', [])
    for k in clusters.keys():
        cluster_names.append(k)

    combo_cluster['values'] = cluster_names
    if len(cluster_names) > 0:
        combo_cluster.current(0)
        # populate the cluster spreadsheet
        row = 1
        for host in clusters[cluster_names[0]]:
            sheet_cluster.set_cell_data(row, 0, host)
            row = row + 1
        cur_cluster_name = cluster_names[0]
        auto_refresh_host_idx = 0
        auto_refresh_sheet_row = 1

    root_window.after(DEFAULT_XBUTIL_REFRESH_INTERVAL*1000, refresh_database, args.json_file)
    root_window.mainloop()


if __name__ == '__main__':
    main()






# -*- coding: utf-8 -*-
# xbutil GUI xbutil Command Handler Module
# Developers: Jim Wu, Annie Ren
# MIT License
# Copyright (c) 2020-2021 xbutil GUI Developers

import subprocess
import json
from xbutil_gui import CU_STATUS_DICT


def get_lspci(host='localhost'):
    lspci_cmd = ['lspci', '-d', '10ee:']
    if host == 'localhost':
        command = lspci_cmd
    else:
        command = ['ssh', host] + lspci_cmd
                   
    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lspci_out = p.stdout.read().decode('utf-8')
    lspci_lines = lspci_out.strip().split('\n')
    lspci_dict = {}
    for l in lspci_lines:
        fields = l.split()
        if len(fields) == 0:
            lspci_dict['NA'] = 'NA'
            break
        device_id = fields[0]
        device_xil_id = fields[-1]
        lspci_dict[device_id] = device_xil_id

    return lspci_dict

def get_xbutil_dump(json_file, host='localhost'):
    xbutil_dump_json = None
    lspci_dict = None
    if json_file is None:
        if host == 'localhost':
            command = ['/opt/xilinx/xrt/bin/unwrapped/xbutil2', 'examine', 
                       '--format', 'JSON', '--report', 'all']
        else:
            command = ['ssh', host, 
                       '/opt/xilinx/xrt/bin/unwrapped/xbutil2', 'examine', 
                       '--format', 'JSON', '--report', 'all']

        #print(' '.join(command))
        p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        xbutil_dump = p.stdout.read()
        try:
            xbutil_dump_json = json.loads(xbutil_dump.decode('utf-8'))
        except json.decoder.JSONDecodeError:
            # xbutil error. maybe card not installed/configured. try lspci
            #print('ERROR: cannot decode json from host', xbutil_dump)
            lspci_dict = get_lspci(host)
            xbutil_dump_json = None
    else:
        with open(json_file, 'r') as fp:
            xbutil_dump_json = json.load(fp)

    return xbutil_dump_json,lspci_dict


def get_devices_compute_units(xbutil_dump_json):
    devices_compute_units = {}
    device_id_names = []
    device_vbnvs = []
    device_ids = []
    compute_units = []
    
    if xbutil_dump_json is None:
        return []

    tmp_devices_vbnvs = {}
    for d in xbutil_dump_json['system']['host']['devices']:
        tmp_devices_vbnvs[d['bdf']] = d['vbnv']

    for d in xbutil_dump_json['devices']:
        device_id = d['device_id']
        device_vbnv = tmp_devices_vbnvs[device_id]
        device_ids.append(device_id)
        device_vbnvs.append(device_vbnv)
        device_id_names.append(device_id + '::' + device_vbnv)
        cur_cu = []
        if isinstance(d['compute_units'], list):
            for cu in d['compute_units']:
                cur_cu.append({'name': cu['name'], 'usage': cu['usage'],
                               'status': CU_STATUS_DICT.get(cu['status']['bit_mask'],
                                                            cu['status']['bit_mask'])})
        else:
            cur_cu.append({'name': 'NA', 'usage': '-', 'status': '-'})

        compute_units.append(cur_cu)

    devices_compute_units['device_ids'] = device_ids
    devices_compute_units['device_vbnvs'] = device_vbnvs
    devices_compute_units['device_id_names'] = device_id_names
    devices_compute_units['compute_units'] = compute_units

    return devices_compute_units


def get_devices_from_lspci(lspci_dict, alveo_spec_dict):
    devices_compute_units = {}
    device_id_names = []
    device_vbnvs = []
    device_ids = []
    compute_units = []

    for k in lspci_dict.keys():
        device_vbnv = alveo_spec_dict['pcie_device_ids'].get(lspci_dict[k], 'NA')
        device_ids.append(k)
        device_vbnvs.append(device_vbnv)
        device_id_names.append(k + "::" + device_vbnv)
        compute_units.append([{'name': 'NA', 'usage': '-', 'status': '-'}])
    
    devices_compute_units['device_ids'] = device_ids
    devices_compute_units['device_vbnvs'] = device_vbnvs
    devices_compute_units['device_id_names'] = device_id_names
    devices_compute_units['compute_units'] = compute_units


    return devices_compute_units


# -*- coding: utf-8 -*-
# xbutil GUI Top Module
# Developers: Jim Wu, Annie Ren
# MIT License
# Copyright (c) 2020-2021 xbutil GUI Developers

import tkinter as tk
from tkinter import messagebox, Toplevel, scrolledtext
import datetime

from xbutil_gui import VERSION


class XbutilTop:
    def __init__(self):
        self.selected_host = None
        self.selected_device_id_name = None
        # top_dicts is a dictionary of dictionary
        # {'host1-device-key1': {top information dictionary for host1},
        #  'host1-device-key2': {top information dictionary for host2},
        #  'host2-device-key1': {top information dictionary for host2}...}
        self.top_dicts = {}
        self.selected_device_id = None
        self.window_top = None
        self.text_top = None

    def generate_top_dict(self, xbutil_dump_json, host, device_id_name):
        host_device_key = host + device_id_name
        if self.top_dicts.get(host_device_key) is None:
            self.top_dicts[host_device_key] = {'device_combo_name': device_id_name,
                                               'compute_units': {},
                                               'dma_metrics' : {}}

        if self.top_dicts[host_device_key].get('compute_units'):
            # save previous data
            for ba in self.top_dicts[host_device_key]['compute_units'].keys():
                self.top_dicts[host_device_key]['compute_units'][ba]['prev_usage'] = \
                    self.top_dicts[host_device_key]['compute_units'][ba]['usage']
                
            for ch in self.top_dicts[host_device_key]['dma_metrics'].keys():
                self.top_dicts[host_device_key]['dma_metrics'][ch]['prev_h2c'] = \
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['h2c']
                self.top_dicts[host_device_key]['dma_metrics'][ch]['prev_c2h'] = \
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['c2h']

        self.top_dicts[host_device_key]['device_memory'] = []
        self.top_dicts[host_device_key]['power'] = 0
        self.top_dicts[host_device_key]['last_updated'] = datetime.datetime.now().strftime("%m/%d/%Y, %H:%M:%S")
        for d in xbutil_dump_json['devices']:
            if d['device_id'] != self.selected_device_id:
                continue

            self.top_dicts[host_device_key]['power'] = d['electrical']['power_consumption_watts']

            # memory topology and usage information
            if d['mem_topology']['board'].get('memory') is not None:
                for m in d['mem_topology']['board']['memory']['memories']:
                    m_dict = {'tag': m['tag'], 'type': m['type'],
                              'temp': int(m['extended_info'].get('temperature_C', 0)),
                              'size': int(m['range_bytes'], 0) >> 20,
                              'usage': int(m['extended_info']['usage']['allocated_bytes'])}

                    self.top_dicts[host_device_key]['device_memory'].append(m_dict)

            # DMA transfer metrics
            for t in d['mem_topology']['board']['direct_memory_accesses']['metrics']:
                ch = t['channel_id']
                h2c = int(t['host_to_card_bytes'], 0)
                c2h = int(t['card_to_host_bytes'], 0)
                if self.top_dicts[host_device_key]['dma_metrics'].get(ch):
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['h2c'] = h2c
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['c2h'] = c2h
                else:
                    self.top_dicts[host_device_key]['dma_metrics'][ch] = \
                        {'h2c': h2c, 'c2h': c2h}

                prev_h2c = self.top_dicts[host_device_key]['dma_metrics'][ch].get('prev_h2c')
                prev_c2h = self.top_dicts[host_device_key]['dma_metrics'][ch].get('prev_c2h')
                if prev_h2c and h2c > prev_h2c:
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['last_h2c'] = \
                        h2c - prev_h2c
                if prev_c2h and c2h > prev_c2h:
                    self.top_dicts[host_device_key]['dma_metrics'][ch]['last_c2h'] = \
                        c2h - prev_c2h

            # compute units
            for c in d['compute_units']:
                ba = c['base_address']
                usage = int(c['usage'])
                if self.top_dicts[host_device_key]['compute_units'].get(ba):
                    self.top_dicts[host_device_key]['compute_units'][ba]['usage'] = usage
                else:
                    self.top_dicts[host_device_key]['compute_units'][ba] = {'usage': usage}

                prev_usage = self.top_dicts[host_device_key]['compute_units'][ba].get('prev_usage')
                if prev_usage and usage > prev_usage:
                    self.top_dicts[host_device_key]['compute_units'][ba]['last_usage'] = \
                        usage - prev_usage

    # show top info
    def show_top_info(self):
        if self.window_top is None or not tk.Toplevel.winfo_exists(self.window_top):
            return

        host_device_key = self.selected_host + self.selected_device_id_name
        if self.top_dicts.get(host_device_key) is None:
            self.text_top.delete('1.0', tk.END)
            self.text_top.insert(tk.END, 'No data is collected yet for host ' + self.selected_host)
            return

        top_dict = self.top_dicts[host_device_key]
        self.text_top.delete('1.0', tk.END)
        self.text_top.insert(tk.END, 'Last updated: ' + top_dict['last_updated'] + '\n')
        self.text_top.insert(tk.END, 'Host: ' + self.selected_host + '\n')
        self.text_top.insert(tk.END, 'Device: ' + top_dict['device_combo_name'] + '\n\n')

        self.text_top.insert(tk.END, 'Device Memory Usage:\n')
        MEM_INFO_HEADER_FORMAT = '{0:12s}|{1:12s}|{2:8s}|{3:12s}|{4:20s}'
        MEM_INFO_FORMAT = '{0:12s}|{1:12s}|{2:<8d}|{3:<12d}|{4:<20d}'

        self.text_top.insert(tk.END,
                        MEM_INFO_HEADER_FORMAT.format('Tag', 'Type', 'Temp', 'Size(MB)', 'Usage'))
        self.text_top.insert(tk.END, '\n')

        for m in top_dict['device_memory']:
            self.text_top.insert(tk.END,
                            MEM_INFO_FORMAT.format(m['tag'], m['type'], m['temp'], m['size'], m['usage']))
            self.text_top.insert(tk.END, '\n')

        self.text_top.insert(tk.END, '\nPower: ' + str(top_dict['power']) + 'W\n')

        self.text_top.insert(tk.END, '\nTotal DMA Transfer Metrics:\n')
        for ch in top_dict['dma_metrics'].keys():
            ch_m = top_dict['dma_metrics'][ch]
            self.text_top.insert(tk.END, 'Channel ' + ch + ': host to card ' +
                            str(ch_m['h2c']) + '\n')
            self.text_top.insert(tk.END, 'Channel ' + ch + ': card to host ' +
                            str(ch_m['c2h']) + '\n')

        self.text_top.insert(tk.END, '\nCompute Unit Usage:\n')
        for ba in top_dict['compute_units'].keys():
            self.text_top.insert(tk.END, 'CU ' + ba + ': ' + 
                                         str(top_dict['compute_units'][ba]['usage']) + '\n')

        self.text_top.insert(tk.END, '\nLast Activity:\n')
        for ba in top_dict['compute_units'].keys():
            last_usage = self.top_dicts[host_device_key]['compute_units'][ba].get('last_usage')
            if last_usage:
                self.text_top.insert(tk.END, 'CU ' + ba + ': ' + str(last_usage) + '\n')

        for ch in top_dict['dma_metrics'].keys():
            last_h2c = top_dict['dma_metrics'][ch].get('last_h2c')
            if last_h2c:
                self.text_top.insert(tk.END, 'Channel ' + ch + ': host to card ' +
                            str(last_h2c) + '\n')
            last_c2h = top_dict['dma_metrics'][ch].get('last_c2h')
            if last_c2h:
                self.text_top.insert(tk.END, 'Channel ' + ch + ': card to host ' +
                            str(last_c2h) + '\n')

        self.text_top.yview(tk.END)

    ###########################################################################
    def show_top_window(self, root_window, selected_host, selected_device_id_name):
        self.selected_host = selected_host
        self.selected_device_id_name = selected_device_id_name
        self.selected_device_id = selected_device_id_name.split('::')[0]

        if self.window_top is not None and tk.Toplevel.winfo_exists(self.window_top):
            return
        else:
            # create a new Toplevel
            self.window_top = Toplevel(root_window)
            self.window_top.title('xbutil GUI ' + VERSION + ' - top')
            self.window_top.geometry('600x850+1000+200')

            cur_grid_row = 0

            self.text_top = scrolledtext.ScrolledText(self.window_top)
            #self.text_top.pack()
            self.text_top.grid(row=cur_grid_row, column=0, sticky='nsew')
            self.window_top.rowconfigure(cur_grid_row, weight=1)
            self.window_top.columnconfigure(0, weight=1)



# -*- coding: utf-8 -*-
#
# xbutil GUI setup.py
# Developers: Jim Wu, Annie Ren
# MIT License
# Copyright (c) 2020-2021 xbutil GUI Developers
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import setuptools
from xbutil_gui import VERSION

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='xbutil_gui', # Replace with your own username
    version=VERSION,
    entry_points={
        'console_scripts' : [
            'xbutil_gui = xbutil_gui.xbutil_gui:main'
        ]
    },
    author="Jim Wu",
    author_email="jimw567@gmail.com",
    description="Xilinx xbutil GUI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jimw567/xbutil-gui",
    packages=['xbutil_gui'],
    install_requires=['pandas', 'matplotlib', 'tksheet'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



# -*- coding: utf-8 -*-
# xbutil GUI Plot Metrics Module
# Developers: Jim Wu, Annie Ren
# MIT License
# Copyright (c) 2020-2021 xbutil GUI Developers

import tkinter as tk
from tkinter import ttk, Toplevel
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from pandas import DataFrame
import datetime

from xbutil_gui import VERSION, COMBO_WIDTH, LABEL_WIDTH, FIGURE_DPI, \
                       DEFAULT_XBUTIL_REFRESH_INTERVAL


class PlotMetrics:
    def __init__(self):
        self.selected_host = None
        self.selected_device_id_name = None
        self.selected_device_id = None

        # members for GUI
        self.label_host_name = None
        self.label_device_id_name = None
        self.window_plot = None
        self.combo_plot_metric = None
        self.combo_plot_auto_refresh = None
        self.figure_hist = None
        self.plot_hist = None
        self.plot_hist_twinx = None
        self.y_ax = None
        self.canvas_hist = None
        self.frame_toolbar = None
        self.toolbar_plot = None

        # members for plot data. each member if a dictionary of dictonary
        # {'host1-device-key1': {metric history},
        #  'host1-device-key2': {metric history},
        #  'host2-device-key1': {metric history}...}
        self.plot_metric = 'power/temperature'
        self.time_hist = {}
        self.power_hist = {}
        self.temp_hist = {}
        self.vccint_hist = {}
        self.iccint_hist = {}

    def get_last_metrics(self, host, device_id_name):
        host_device_key = host + device_id_name
        if self.power_hist.get(host_device_key) is not None:
            power = self.power_hist[host_device_key][-1]
            temp = self.temp_hist[host_device_key][-1]
        else:
            power = 0
            temp = 0

        return power, temp

    def plot_metrics(self, auto_refresh_seconds):
        if not (self.window_plot is not None and tk.Toplevel.winfo_exists(self.window_plot)):
            return

        selected_plot_metric = self.combo_plot_metric.get()
        if self.combo_plot_auto_refresh.get() == 'off':
            return
        auto_refresh_interval = int(self.combo_plot_auto_refresh.get())
        if selected_plot_metric != self.plot_metric:
            # always plot when selected metric is changed.
            pass
        elif auto_refresh_seconds % auto_refresh_interval > 0:
            return

        index_step = int(auto_refresh_interval/DEFAULT_XBUTIL_REFRESH_INTERVAL)
        host_device_key = self.selected_host + self.selected_device_id_name
        self.plot_metric = selected_plot_metric
        if len(self.power_hist[host_device_key][::index_step]) == 0:
            # nothing to plot yet
            return

        self.plot_hist.clear()
        if self.plot_hist_twinx is not None:
            self.plot_hist_twinx.clear()

        if self.plot_metric == 'power/temperature':
            y_hist_dict = {'time': self.time_hist[host_device_key][::index_step],
                           'power': self.power_hist[host_device_key][::index_step],
                           'temp': self.temp_hist[host_device_key][::index_step]}
            y_hist_df = DataFrame(y_hist_dict, columns=['time', 'power', 'temp'])
            y_hist_df.plot(kind='line', legend=True, x='time', y='power',
                           ax=self.plot_hist, color='r', marker='.', fontsize=10)
            self.plot_hist_twinx = y_hist_df['temp'].plot(
                kind='line', legend=True, ax=self.plot_hist,
                color='b', marker='.', fontsize=10, secondary_y=True)
            self.plot_hist.set_ylabel('Power(W)')
            self.plot_hist.right_ax.set_ylabel('Temperature (C)')
            self.plot_hist.set_title('Power(w)/Temperature(C) History')
            self.canvas_hist.draw()
        elif self.plot_metric == 'vccint/iccint':
            y_hist_dict = {'time': self.time_hist[host_device_key][::index_step],
                           'vccint': self.vccint_hist[host_device_key][::index_step],
                           'iccint': self.iccint_hist[host_device_key][::index_step]}
            y_hist_df = DataFrame(y_hist_dict, columns=['time', 'vccint', 'iccint'])
            y_hist_df.plot(kind='line', legend=True, x='time', y='vccint', ax=self.plot_hist,
                           color='r', marker='.', fontsize=10)
            self.plot_hist_twinx = y_hist_df['iccint'].plot(
                kind='line', legend=True, ax=self.plot_hist,
                color='b', marker='.', fontsize=10, secondary_y=True)
            self.plot_hist.set_ylabel('Vccint(V)')
            self.plot_hist.right_ax.set_ylabel('Iccint(A)')
            self.plot_hist.set_title('Vccint(V)/Iccint(A) History')
            self.canvas_hist.draw()

    def show_plot_window(self, root_window, selected_host, selected_device_id_name):
        self.selected_host = selected_host
        self.selected_device_id_name = selected_device_id_name
        self.selected_device_id = selected_device_id_name.split('::')[0]

        if self.window_plot is not None and tk.Toplevel.winfo_exists(self.window_plot):
            self.label_host_name['text'] = selected_host
            #self.label_host_name['text'] = 'host1'  # for documentation purpose
            self.label_device_id_name['text'] = selected_device_id_name
            return

        self.window_plot = Toplevel(root_window)
        self.window_plot.title('xbutil GUI ' + VERSION + ' - plot')

        cur_grid_row = 0
        label_host = ttk.Label(self.window_plot, text="Host", width=LABEL_WIDTH, anchor='e')
        label_host.grid(row=cur_grid_row, column=0, sticky='e')
        self.label_host_name = ttk.Label(self.window_plot, text=selected_host, width=40, anchor='w')
        self.label_host_name.grid(row=cur_grid_row, column=1, sticky='w')
        cur_grid_row = cur_grid_row + 1

        label_device = ttk.Label(self.window_plot, text="Device", width=LABEL_WIDTH, anchor='e')
        label_device.grid(row=cur_grid_row, column=0, sticky='e')
        self.label_device_id_name = ttk.Label(self.window_plot, text=selected_device_id_name, width=40, anchor='w')
        self.label_device_id_name.grid(row=cur_grid_row, column=1, sticky='w')
        cur_grid_row = cur_grid_row + 1

        # Add a dropdown list for plot metric
        lable_plot_metric = ttk.Label(self.window_plot, text="Plot metric").grid(
            row=cur_grid_row, column=0, sticky='e')
        self.combo_plot_metric = ttk.Combobox(self.window_plot, width=COMBO_WIDTH)
        self.combo_plot_metric['values'] = ('power/temperature', 'vccint/iccint')
        self.combo_plot_metric.grid(row=cur_grid_row, column=1, sticky='w')
        cur_grid_row = cur_grid_row + 1
        self.combo_plot_metric.current(0)

        # Auto refresh dropdown
        label_plot_auto_refresh = ttk.Label(self.window_plot, text="Auto Refresh interval(s)").grid(
            row=cur_grid_row, column=0, sticky='e')
        self.combo_plot_auto_refresh = ttk.Combobox(self.window_plot, width=COMBO_WIDTH)
        self.combo_plot_auto_refresh['values'] = ('off', '10', '60')
        self.combo_plot_auto_refresh.grid(row=cur_grid_row, column=1, sticky='w')
        self.combo_plot_auto_refresh.current(1)
        cur_grid_row = cur_grid_row + 1

        # plot row
        self.figure_hist = plt.Figure(figsize=(10, 5), dpi=FIGURE_DPI)
        self.plot_hist = self.figure_hist.add_subplot(111)
        self.canvas_hist = FigureCanvasTkAgg(self.figure_hist, self.window_plot)
        self.canvas_hist.get_tk_widget().grid(row=cur_grid_row, columnspan=4, sticky='nsew')
        cur_grid_row = cur_grid_row + 1

        # Plot navigation toolbar
        self.frame_toolbar = tk.Frame(self.window_plot)
        self.frame_toolbar.grid(row=cur_grid_row, columnspan=4)
        self.toolbar_plot = NavigationToolbar2Tk(self.canvas_hist, self.frame_toolbar)
        cur_grid_row = cur_grid_row + 1

    def update_history(self, xbutil_dump_json, host, device_id_name):
        host_device_key = host + device_id_name
        device_id = device_id_name.split('::')[0]
        if self.time_hist.get(host_device_key) is None:
            self.time_hist[host_device_key] = []
            self.power_hist[host_device_key] = []
            self.temp_hist[host_device_key] = []
            self.vccint_hist[host_device_key] = []
            self.iccint_hist[host_device_key] = []

        fpga_present = False
        for dev in xbutil_dump_json['devices']:
            if dev['device_id'] != device_id:
                continue

            for t in dev['thermals']:
                if t['location_id'] == 'fpga0' and t['is_present']:
                    fpga_present = True
                    fpga_temp = float(t['temp_C'])

        if not fpga_present:
            return

        self.time_hist[host_device_key].append(datetime.datetime.now().strftime("%m/%d %H:%M"))
        self.temp_hist[host_device_key].append(fpga_temp)

        for dev in xbutil_dump_json['devices']:
            if dev['device_id'] != device_id:
                continue

            self.power_hist[host_device_key].append(float(dev['electrical']['power_consumption_watts']))
            for pr in dev['electrical']['power_rails']:
                if pr['id'] == 'vccint':
                    self.vccint_hist[host_device_key].append(float(pr['voltage']['volts']))
                    self.iccint_hist[host_device_key].append(float(pr['current']['amps']))



# -*- coding: utf-8 -*-

from pathlib import Path
import sys

# Make sure it's running in Python 3.6 as it's required by tksheet.
if sys.version_info[0] < 2 or (sys.version_info[0] == 3 and sys.version_info[1] < 6):
    sys.exit('ERROR: This program require Python 3.6 or newer. You are running %s' %
             '.'.join(map(str, sys.version_info[0:3])))

VERSION = '0.0.6'
LABEL_WIDTH = 15
COMBO_WIDTH = 40
FIGURE_DPI = 100
DEFAULT_XBUTIL_REFRESH_INTERVAL = 5

# sheet column numbers
SHEET_TOTAL_ROWS = 200
SHEET_TOTAL_COLS = 8
SHEET_HOST_COL = 0
SHEET_DEVICE_COL = 1
SHEET_CU_COL = 2
SHEET_CU_STATUS_COL = 3
SHEET_CU_USAGE_COL = 4
SHEET_POWER_COL = 5
SHEET_TEMP_COL = 6
SHEET_LAST_UPDATED_COL = 7

CU_STATUS_DICT = {
    "0x1": "START",
    "0x4": "IDLE", 
    "0x8": "READY",
    "0x10": "RESTART"
}

__resource_path__ = Path(__file__).parent / 'resources'
__icon__ = __resource_path__ / 'xbutil-icon.gif'

STATUS_CODES = {
    'XRT_NOT_SETUP': {
        'code': 1,
        'message': 'ERROR: xbutil not found. Please set up XRT environment before running this application.'
    }
}
